@@ -8,7 +8,7 @@ use Exporter;
 use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use strict;
 use warnings;
-#use Smart::Comments;
+#use Smart::Comments q(#####);	# Three # for "I am here" messages, four # for dumps.
 
 @ISA = qw(Exporter);
 
@@ -40,13 +40,17 @@ use warnings;
 	) ],
 	'utility' => [ qw(
 		epsilon
+		fltcmp
 		poly_iteration
+		poly_tolerance
+		newtonraphson
 		laguerre
 		poly_antiderivative
 		poly_derivative
 		poly_constmult
 		poly_divide
 		poly_evaluate
+		poly_derivaluate
 		poly_nonzero_term_count
 		simplified_form
 	) ],
@@ -55,7 +59,7 @@ use warnings;
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'classical'} }, @{ $EXPORT_TAGS{'numeric'} },
 	@{ $EXPORT_TAGS{'sturm'} }, @{ $EXPORT_TAGS{'utility'} } );
 
-our $VERSION = '2.55_4';
+our $VERSION = '2.62_1';
 
 #
 # Options to set or unset to force poly_roots() to use different
@@ -86,16 +90,19 @@ my %option = (
 #
 my %iteration = (
 	hessenberg => 60,
+	newtonraphson => 60,
 	laguerre => 60,
 	sturm_bisection => 100,
 );
 
 #
-# Values here are placeholders only. The actual values will get
-# set in the BEGIN block.
+# Some values here are placeholders only, and will get
+# replaced in the BEGIN block.
 #
 my %tolerance = (
+	newtonraphson => 1e-14,
 	laguerre => 1e-14,
+	fltcmp => 1.5e-8,
 );
 
 #
@@ -117,6 +124,7 @@ BEGIN
 	}
 
 	$tolerance{laguerre} = 2 * $epsilon;
+	$tolerance{newtonraphson} = 2 * $epsilon;
 }
 
 #
@@ -125,9 +133,25 @@ BEGIN
 #
 sub sign
 {
-	my($x) = @_;
-	return 1 if ($x > 0);
-	return -1 if ($x < 0);
+	return 1 if ($_[0] > 0);
+	return -1 if ($_[0] < 0);
+	return 0;
+}
+
+#
+# if (fltcmp($x, $y) == 0) { ... }
+#
+# Compare two floating point numbers to a certain degree of accuracy.
+# Like other functions ending in "cmp", returns -1 if $x < $y, 1 if
+# $x > $y, and 0 if the arguments are equal, the difference being that
+# the comparisons are made within a tolerance set in poly_tolerance().
+#
+sub fltcmp
+{
+	my($a, $b) = @_;
+
+	return -1 if ($a + $tolerance{fltcmp} < $b);
+	return 1 if ($a - $tolerance{fltcmp} > $b);
 	return 0;
 }
 
@@ -152,11 +176,13 @@ sub epsilon
 #
 sub get_hessenberg
 {
+	carp "get_hessenberg() is DEPRECATED. Please use \"\%opts = poly_option(); \$opts{hessenberg};\" instead.";
 	return $option{hessenberg};
 }
 
 sub set_hessenberg
 {
+	carp "set_hessenberg() is DEPRECATED. Please use \"poly_option(hessenberg => $_[0]);\" instead.";
 	$option{hessenberg} = ($_[0])? 1: 0;
 }
 
@@ -189,6 +215,37 @@ sub poly_option
 	return %old_opts;
 }
 
+#
+# poly_tolerance(opt1 => n, opt2 => n, ...);
+#
+sub poly_tolerance
+{
+	my %tols = @_;
+	my %old_tols;
+
+	return %tolerance if (scalar @_ == 0);
+
+	foreach my $k (keys %tols)
+	{
+		#
+		# If this is a real tolerance limit, save its old value, then set it.
+		#
+		if (exists $tolerance{$k})
+		{
+			my $val = abs($tols{$k});
+
+			$old_tols{$k} = $tolerance{$k};
+			$tolerance{$k} = $val;
+		}
+		else
+		{
+			croak "poly_tolerance(): unknown key $k.";
+		}
+	}
+
+	return %old_tols;
+}
+
 #
 # poly_iteration(opt1 => n, opt2 => n, ...);
 #
@@ -857,7 +914,7 @@ sub hqr_eigen_hessenberg
 				}
 
 				my $j = $k + 3;
-				$j = $n if $j > $n;
+				$j = $n if ($j > $n);
 
 				#
 				# Column modification.
@@ -1007,10 +1064,6 @@ sub poly_analysis
 	# the first three prime numbers, but it's not much of a waste of
 	# space to go up to 31.
 	#
-	#for my $p (@primeset)
-	#{
-	#push @czp, $p if (($#coefficients % $p) == 0);
-	#}
 	@czp = grep(($#coefficients % $_) == 0,
 		(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31)
 	);
@@ -1036,7 +1089,7 @@ sub poly_analysis
 		#
 		map(($m *= $_), @czp);
 
-		### Substitution degree: $m
+		#### Substitution degree: $m
 	}
 
 	#
@@ -1178,9 +1231,69 @@ sub poly_evaluate
 			$results[$j] = $results[$j] * $values[$j] + $c;
 		}
 	}
+
 	return wantarray? @results: $results[0];
 }
 
+#
+# ($y, $dy, $ddy) = poly_derivaluate(\@coefficients, $x);
+#
+# Returns p(x), p'(x), and p"(x) of the polynomial for an
+# x-value, using Horner's method.
+#
+sub poly_derivaluate
+{
+	my($coef_ref, $x) = @_;
+	my(@coefficients) = @$coef_ref;
+	my $n = $#coefficients;
+	my $val = shift @coefficients;
+	my $d1val = $val * $n;
+	my $d2val = 0;
+
+	#
+	# Be nice and check if the user accidentally passed in
+	# a reference for the $x value.
+	##### poly_derivaluate
+	##### $coef_ref
+	##### $x
+	#
+	croak "Used a reference instead of an X value in poly_derivaluate()" if (ref $x eq "ARRAY" or ref $x eq "SCALAR");
+
+	#
+	# Special case for the linear eq'n (the y = constant eq'n
+	# takes care of itself).
+	#
+	if ($n == 1)
+	{
+		$d1val = $val;
+		$val = $val * $x + $coefficients[0];
+	}
+	elsif ($n >= 2)
+	{
+		my $lastn = --$n;
+		$d2val = $d1val * $n;
+
+		#
+		# Loop through the coefficients, except for
+		# the linear and constant terms.
+		#
+		foreach my $c (@coefficients[0..$lastn-2])
+		{
+			$val = $val * $x + $c;
+			$d1val = $d1val * $x + ($c *= $n--);
+			$d2val = $d2val * $x + $c;
+		}
+
+		#
+		# Handle the last two coefficients.
+		#
+		$d1val = $d1val * $x + $coefficients[$lastn-1];
+		$val = ($val * $x + $coefficients[$lastn-1]) * $x + $coefficients[$lastn];
+	}
+
+	return ($val, $d1val, $d2val);
+}
+
 #
 # ($q_ref, $r_ref) = poly_divide(\@coefficients1, \@coefficients2);
 #
@@ -1210,10 +1323,10 @@ sub poly_divide
 	return (undef, undef) if ($d_degree < 0);
 
 	#
-	###Dividing:
-	### @numerator
-	### by
-	### @divisor
+	### poly_divide():
+	#### @numerator
+	#### by
+	#### @divisor
 	#
 	my $lead_coefficient = $divisor[0];
 
@@ -1289,7 +1402,7 @@ sub poly_sturm_chain
 	while ($#$r > 0);
 
 	#
-	#### poly_sturm_chain:
+	### poly_sturm_chain:
 	#### @chain
 	#
 	return @chain;
@@ -1316,8 +1429,8 @@ sub poly_real_root_count
 # $root_count = sturm_real_root_range_count(\@chain, $x0, $x1);
 #
 # An all-in-one function for finding the number of real roots in
-# a polynomial. Use this if you don't intend to do anything else
-# requiring the Sturm chain.
+# a polynomial over a range in X. Use this if you don't intend to do
+# anything else requiring the Sturm chain.
 #
 sub sturm_real_root_range_count
 {
@@ -1380,12 +1493,12 @@ sub sturm_bisection_roots
 			my $midto_count = sturm_real_root_range_count($chain_ref, $mid, $to);
 
 			#
-			### $its
-			### $from
-			### $to
-			### $mid
-			### $frommid_count
-			### $midto_count
+			#### $its
+			#### $from
+			#### $to
+			#### $mid
+			#### $frommid_count
+			#### $midto_count
 			#
 
 			#
@@ -1523,14 +1636,16 @@ sub sturm_sign_count
 	return $scnt;
 }
 
+#
+# @roots = laguerre(\@coefficients, \@xvals);
+#
+# Find the roots nearby the given X values.
+#
 sub laguerre
 {
 	my($p_ref, $xval_ref) = @_;
-	my(@p0) = @$p_ref;
-	my(@p1) = poly_derivative(@p0);
-	my(@p2) = poly_derivative(@p1);
-	my $n = $#p0;
-	my @values;
+	my $n = $#$p_ref;
+	my @xvalues;
 	my @roots;
 
 	#
@@ -1538,49 +1653,56 @@ sub laguerre
 	#
 	if (ref $xval_ref eq "ARRAY")
 	{
-		@values = @$xval_ref;
+		@xvalues = @$xval_ref;
 	}
 	else
 	{
 		#
 		# It could happen. Someone might type \$x instead of $x.
 		#
-		@values = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
+		@xvalues = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
 	}
 
-	foreach my $x (@values)
+	foreach my $x (@xvalues)
 	{
+		#
+		### laguerre: looking near $x
+		### Coefficient: @$p_ref
+		### Degree: $n
+		#
 		my $its = 0;
 
 		ROOT:
-		for(;;)
+		for (;;)
 		{
 			#
-			# There's a better way to do this, but
-			# I'm going for "simple to code" for
-			# this developer release.
+			# Get the values of the function and its first and
+			# second derivatives at X.
 			#
-			my $valp0 = poly_evaluate($p_ref, $x);
-			my $valp1 = poly_evaluate(\@p1, $x);
-			my $valp2 = poly_evaluate(\@p2, $x);
+			my($y, $dy, $d2y) = poly_derivaluate($p_ref, $x);
 
-			if (abs($valp0) <= $tolerance{laguerre})
+			if (abs($y) <= $tolerance{laguerre})
 			{
 				push @roots, $x;
 				last ROOT;
 			}
 
 			#
-			### $its
-			### $x
-			### $valp0
-			### $valp1
-			### $valp2
+			#### At Iteration: $its
+			#### X: $x
+			#### f(x): $y
+			#### f'(x): $dy
+			#### f''(x): $d2y
 			#
-			my $g = $valp1/$valp0;
-			my $h = $g * $g - $valp2/$valp0;
+			my $g = $dy/$y;
+			my $h = $g * $g - $d2y/$y;
 			my $f = sqrt(($n - 1) * ($n * $h - $g*$g));
+			$f = - $f if (abs($g - $f) > abs($g + $f));
 
+			#
+			#### g = $g
+			#### h = $h
+			#### f = $f
 			#
 			# Divide by the largest value of $g plus
 			# $f, bearing in mind that $f is the result
@@ -1590,7 +1712,6 @@ sub laguerre
 			# Use the abs() function to determine size
 			# since $g or $f may be complex numbers.
 			#
-			$f = - $f if (abs($g - $f) > abs($g + $f));
 			my $dx = $n/($g + $f);
 
 			$x -= $dx;
@@ -1603,6 +1724,78 @@ sub laguerre
 			croak "Too many iterations ($its) at dx=$dx\n" if ($its >= $iteration{laguerre});
 			$its++;
 		}
+
+		### root found at iteration $its
+		#### $x
+	}
+	return @roots;
+}
+
+#
+# @xvals = newtonraphson(\@coefficients, \@xvals);
+#
+# Find the roots nearby the given X values.
+#
+sub newtonraphson
+{
+	my($p_ref, $xval_ref) = @_;
+	my $n = $#$p_ref;
+	my @xvalues;
+	my @roots;
+
+	#
+	# Allow some flexibility in sending the x-values.
+	#
+	if (ref $xval_ref eq "ARRAY")
+	{
+		@xvalues = @$xval_ref;
+	}
+	else
+	{
+		#
+		# It could happen. Someone might type \$x instead of $x.
+		#
+		@xvalues = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
+	}
+
+	#
+	### newtonraphson()
+	#### @xvalues
+	#
+	foreach my $x (@xvalues)
+	{
+		my $its = 0;
+
+		ROOT:
+		for (;;)
+		{
+			#
+			# Get the values of the function and its first and
+			# second derivatives at X.
+			#
+			my($y, $dy, $d2y) = poly_derivaluate($p_ref, $x);
+			my $dx = $y/$dy;
+			$x -= $dx;
+
+			if (abs($dx) <= $tolerance{newtonraphson})
+			{
+				push @roots, $x;
+				last ROOT;
+			}
+
+			#
+			#### At Iteration: $its
+			#### X: $x
+			#### f(x): $y
+			#### f'(x): $dy
+			#### f''(x): $d2y
+			#
+			croak "Too many iterations ($its) at dx=$dx\n" if ($its >= $iteration{newtonraphson});
+			$its++;
+		}
+
+		### root found at iteration $its
+		#### $x
 	}
 	return @roots;
 }
@@ -1612,27 +1805,6 @@ sub poly_gcd
 	my($c1_ref, $c2_ref) = @_;
 }
 
-sub list_from_reference
-{
-	my($xval_ref) = @_;
-	my @values;
-
-	#
-	# Allow some flexibility in sending the x-values.
-	#
-	if (ref $xval_ref eq "ARRAY")
-	{
-		@values = @$xval_ref;
-	}
-	else
-	{
-		#
-		# It could happen. Someone might type \$x instead of $x.
-		#
-		@values = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
-	}
-	return @values;
-}
 
 1;
 __END__
@@ -1712,6 +1884,8 @@ or
 This package supplies a set of functions that find the roots of
 polynomials, along with some utility functions.
 
+Roots will be either real or of type L<Math::Complex>.
+
 Functions making use of the Sturm sequence are also available, letting you
 find the number of real roots present in a range of X values.
 
@@ -1723,70 +1897,6 @@ also been exported for your use.
 These are the functions that calculate the polynomial's roots through numeric
 algorithms. They are all exported under the tag "numeric".
 
-=head3 poly_option()
-
-Set options that affect the behavior of the C<poly_roots()> function. All options
-are set to either 1 ("on") or 0 ("off").
-
-This is the option function that deprecates C<set_hessenberg()> and
-C<get_hessenberg()>.
-
-Options may be set and saved:
-
-    #
-    # Set a few options...
-    #
-    poly_option(hessenberg => 0, root_function => 1);
-
-    #
-    # Get all of the current options and their values.
-    #
-    my %all_options = poly_option();
-
-    #
-    # Set some options but save the former option values
-    # for later.
-    #
-    my %changed_options = poly_option(hessenberg => 1, varsubst => 1);
-
-The current options available are:
-
-=over 4
-
-=item hessenberg
-
-Use the QR Hessenberg matrix method to solve the polynomial. By default, this
-is set to 1. If set to 0, C<poly_roots()> uses one of the classical
-root-finding functions listed below, I<if> the degree of the polynomial is four
-or less.
-
-=item root_function
-
-If the polynomial is of the form C<ax**n + c>, poly_roots() will make use of the
-L<root()|Math::Complex/OPERATIONS> function from Math::Complex. This takes
-precedence over the other solving methods.
-
-=item varsubst
-
-Reduce polynomials to a lower degree through variable substitution, if possible.
-
-For example, with varsubst set to one and the polynomial to solve is C<9x**6 + 128x**3 + 21>,
-then poly_roots() will reduce the polynomial to C<9y**2 + 128y + 21>, where y = x**3,
-and solve the quadratic (either classically or numerically, depending
-on the hessenberg option). Taking the cube root of each quadratic root
-completes the operation.
-
-This has the benefit of having a simpler matrix to solve, or if the hessenberg option
-is set to zero, has the effect of being able to use one of the classical methods on a
-polynomial of high degree. In the above example, the order-six polynomial gets solved
-with the quadratic_roots() function if the hessenberg option is zero.
-
-Currently the variable substitution is fairly simple and will only look
-for gaps of zeros in the coefficients that are multiples of the prime numbers
-less than or equal to 31 (2, 3, 5, et cetera).
-
-=back
-
 =head3 poly_roots()
 
 Returns the roots of a polynomial equation, regardless of degree.
@@ -1817,6 +1927,71 @@ This is a complete change from the default behavior in versions less than v2.50.
 
 B<NOTE>: this function is replaced by the option function C<poly_option()>.
 
+=head3 poly_option()
+
+Set options that affect the behavior of the C<poly_roots()> function. All options
+are set to either 1 ("on") or 0 ("off"). See also L</poly_iteration()>
+and L</poly_tolerance()>.
+
+This is the option function that deprecates C<set_hessenberg()> and
+C<get_hessenberg()>.
+
+Options may be set and saved:
+
+  #
+  # Set a few options...
+  #
+  poly_option(hessenberg => 0, root_function => 1);
+
+  #
+  # Get all of the current options and their values.
+  #
+  my %all_options = poly_option();
+
+  #
+  # Set some options but save the former option values
+  # for later.
+  #
+  my %changed_options = poly_option(hessenberg => 1, varsubst => 1);
+
+The current options available are:
+
+=over 4
+
+=item hessenberg
+
+Use the QR Hessenberg matrix method to solve the polynomial. By default, this
+is set to 1. If set to 0, C<poly_roots()> uses one of the L<classical|Classical Functions>
+root-finding functions listed below, I<if> the degree of the polynomial is four
+or less.
+
+=item root_function
+
+Use the L<root()|Math::Complex/OPERATIONS> function from Math::Complex if the
+polynomial is of the form C<ax**n + c>. This will take precedence over the other
+solving methods.
+
+=item varsubst
+
+Reduce polynomials to a lower degree through variable substitution, if possible.
+
+For example, with varsubst set to one and the polynomial to solve is C<9x**6 + 128x**3 + 21>,
+then C<poly_roots()> will reduce the polynomial to C<9y**2 + 128y + 21>, where y = x**3,
+and solve the quadratic (either classically or numerically, depending
+on the hessenberg option). Taking the cube root of each quadratic root
+completes the operation.
+
+This has the benefit of having a simpler matrix to solve, or if the hessenberg option
+is set to zero, has the effect of being able to use one of the classical methods on a
+polynomial of high degree. In the above example, the order-six polynomial gets solved
+with the quadratic_roots() function if the hessenberg option is zero.
+
+Currently the variable substitution is fairly simple and will only look
+for gaps of zeros in the coefficients that are multiples of the prime numbers
+less than or equal to 31 (2, 3, 5, et cetera).
+
+=back
+
 =head2 Classical Functions
 
 These are the functions that solve polynomials via the classical methods.
@@ -1834,7 +2009,7 @@ functions.
 Here for completeness's sake more than anything else. Returns the
 solution for
 
-    ax + b = 0
+  ax + b = 0
 
 by returning C<-b/a>. This may be in either a scalar or an array context.
 
@@ -1842,7 +2017,7 @@ by returning C<-b/a>. This may be in either a scalar or an array context.
 
 Gives the roots of the quadratic equation
 
-    ax**2 + bx + c = 0
+  ax**2 + bx + c = 0
 
 using the well-known quadratic formula. Returns a two-element list.
 
@@ -1850,9 +2025,9 @@ using the well-known quadratic formula. Returns a two-element list.
 
 Gives the roots of the cubic equation
 
-    ax**3 + bx**2 + cx + d = 0
+  ax**3 + bx**2 + cx + d = 0
 
-by the method described by R. W. D. Nickalls (see the L</"ACKNOWLEDGMENTS">
+by the method described by R. W. D. Nickalls (see the L</ACKNOWLEDGMENTS>
 section below). Returns a three-element list. The first element will
 always be real. The next two values will either be both real or both
 complex numbers.
@@ -1861,9 +2036,9 @@ complex numbers.
 
 Gives the roots of the quartic equation
 
-    ax**4 + bx**3 + cx**2 + dx + e = 0
+  ax**4 + bx**3 + cx**2 + dx + e = 0
 
-using Ferrari's method (see the L</"ACKNOWLEDGMENTS"> section below). Returns
+using Ferrari's method (see the L</ACKNOWLEDGMENTS> section below). Returns
 a four-element list. The first two elements will be either
 both real or both complex. The next two elements will also be alike in
 type.
@@ -1877,21 +2052,20 @@ They are all exported under the tag "sturm".
 
 Return the number of I<unique>, I<real> roots of the polynomial.
 
-    $unique_roots = poly_real_root_count(@coefficients);
+  $unique_roots = poly_real_root_count(@coefficients);
 
 For example, the equation C<(x + 3)**3> forms the polynomial C<x**3 + 9x**2 + 27x + 27>,
-but C<poly_real_root_count(1, 9, 27, 27)> will return 1 since all
-three roots are identical.
+but since all three of its roots are identical, C<poly_real_root_count(1, 9, 27, 27)> will return 1.
 
 Likewise, C<poly_real_root_count(1, -8, 25)> will return 0 because the two roots
 of C<x**2 -8x + 25> are both complex.
 
 This function is the all-in-one function to use instead of
 
-    my @chain = poly_sturm_chain(@coefficients);
+  my @chain = poly_sturm_chain(@coefficients);
 
-    return sturm_sign_count(sturm_sign_minus_inf(\@chain)) -
-            sturm_sign_count(sturm_sign_plus_inf(\@chain));
+  return sturm_sign_count(sturm_sign_minus_inf(\@chain)) -
+          sturm_sign_count(sturm_sign_plus_inf(\@chain));
 
 if you don't intend to use the Sturm chain for anything else.
 
@@ -1908,40 +2082,85 @@ See L</poly_real_root_count()> above for an example of its use.
 
 =head3 sturm_real_root_range_count()
 
-Return the number of I<unique>, I<real> roots of the polynomial.
+Return the number of I<unique>, I<real> roots of the polynomial between two X values.
 
-    my($x0, $x1) = (0, 1000);
+  my($x0, $x1) = (0, 1000);
 
-    my @chain = poly_sturm_chain(@coefficients);
-    $unique_roots = sturm_real_root_range_count(\@chain, $x0, $x1);
+  my @chain = poly_sturm_chain(@coefficients);
+  $unique_roots = sturm_real_root_range_count(\@chain, $x0, $x1);
 
-Internally, this is equivalent to:
+This is equivalent to:
 
-    my @signs = sturm_sign_chain(\@chain, [$x0, $x1]);
-    return sturm_sign_count(@{$signs[0]}) - sturm_sign_count(@{$signs[1]});
+  my($x0, $x1) = (0, 1000);
 
-=head3 sturm_sign_minus_inf()
+  my @chain = poly_sturm_chain(@coefficients);
+  my @signs = sturm_sign_chain(\@chain, [$x0, $x1]);
+  $unique_roots = sturm_sign_count(@{$signs[0]}) - sturm_sign_count(@{$signs[1]});
 
-=head3 sturm_sign_plus_inf()
+=head3 Sturm Sign Sequence Functions
 
-=head3 sturm_sign_chain()
+=head4 sturm_sign_chain()
 
-The functions that return the array of signs that are used by the functions
+=head4 sturm_sign_minus_inf()
+
+=head4 sturm_sign_plus_inf()
+
+These functions return the array of signs that are used by the functions
 L</poly_real_root_count()> and L</sturm_real_root_range_count()> to find
-the number of real roots in a polynomial. C<sturm_sign_minus_inf()>
-and C<sturm_sign_plus_inf()> return a single array for the values at minus
-infinity and plus infinity (for open-ended ranges, e.g., "all roots less than zero").
-C<sturm_sign_chain()> returns an array of arrays of signs, one for each X value
-passed into the function.
+the number of real roots in a polynomial.
+
+In normal use you will probably never need to use them, unless you want
+to examine the internals of the Sturm functions:
+
+  #
+  # Examine the sign changes that occur at each endpoint of
+  # the x range.
+  #
+  my(@coefficients) = (1, 4, 7, 23);
+  my(@xvals) = (-12, 12);
+
+  my @chain = poly_sturm_chain( @coefficients);
+  my @signs = sturm_sign_chain(\@chain, \@xvals);  # An array of arrays.
+
+  print "\nPolynomial: [", join(", ", @coefficients), "]\n";
+
+  foreach my $j (0..$#signs)
+  {
+    my @s = @{$signs[$j]};
+    print $xval[$j], "\n",
+          "\t", join(", ", @s), "], sign count = ",
+          sturm_sign_count(@s), "\n\n";
+    }
+
+Similar examinations can be made at plus and minus infinity:
+
+  #
+  # Examine the sign changes that occur between plus and minus
+  # infinity.
+  #
+  my @coefficients = (1, 4, 7, 23);
+
+  my @chain = poly_sturm_chain( @coefficients);
+  my @smi = sturm_sign_minus_inf(\@chain);
+  my @spi = sturm_sign_plus_inf(\@chain);
+
+  print "\nPolynomial: [", join(", ", @coefficients), "]\n";
+
+  print "Minus Inf:\n",
+        "\t", join(", ", @smi), "], sign count = ",
+        sturm_sign_count(@smi), "\n\n";
+
+  print "Plus Inf:\n",
+        "\t", join(", ", @spi), "], sign count = ",
+        sturm_sign_count(@spi), "\n\n";
 
 =head3 sturm_sign_count()
 
 Counts and returns the number of sign changes in a sequence of signs,
-such as those returned by L</sturm_sign_minus_inf()>, L</sturm_sign_plus_inf()>,
-and L</sturm_sign_chain()>
+such as those returned by the L</Sturm Sign Sequence Functions>
 
-See L</poly_real_root_count()> and L</sturm_real_root_range_count()> for examples
-of its use.
+See L</poly_real_root_count()> and L</sturm_real_root_range_count()> for
+examples of its use.
 
 =head3 sturm_bisection_roots()
 
@@ -1949,12 +2168,14 @@ Return the I<real> roots counted by L</sturm_real_root_range_count()>. Uses the
 bisection method combined with C<sturm_real_range_count()> to narrow the range
 to a single root, then uses L</laguerre()> to find the value.
 
-    my($from, $to) = (-1000, 0);
-    my @chain = poly_sturm_chain(@coefficients);
-    my @roots = sturm_bisection_roots(\@chain, $from, $to);
+  my($from, $to) = (-1000, 0);
+  my @chain = poly_sturm_chain(@coefficients);
+  my @roots = sturm_bisection_roots(\@chain, $from, $to);
 
 As it is using the Sturm functions, it will find only the real roots.
 
+Internally, laguerre() is used by sturm_bisection_roots().
+
 =head2 Utility Functions
 
 These are internal functions used by the other functions listed above,
@@ -1966,70 +2187,158 @@ Returns the machine epsilon value that was calculated when this module was loade
 
 The value may be changed, although this in general is not recommended.
 
-    my $old_epsilon = epsilon($new_epsilon);
+  my $old_epsilon = epsilon($new_epsilon);
 
 The previous value of epsilon may be saved to be restored later.
 
 The Wikipedia article at L<http://en.wikipedia.org/wiki/Machine_epsilon/> has
 more information on the subject.
 
+=head3 fltcmp()
+
+Compare two floating point numbers within a degree of accuracy.
+
+Like most functions ending in "cmp", this one returns -1 if the first
+argument tests as less than the second argument, 1 if the first tests
+greater than the second, and 0 otherwise. Comparisons are made within
+a tolerance range that may be set with L</poly_tolerance()>.
+
+  #
+  # Set a very forgiving comparison tolerance.
+  #
+  poly_tolerance(fltcmp => 1e-5);
+  my @x = poly_roots(@cubic);
+  my @y = poly_evaluate(\@cubic, \@x);
+
+  if (fltcmp($y[0], 0.0) == 0 and
+      fltcmp($y[1], 0.0) == 0 and
+      fltcmp($y[2], 0.0) == 0)
+  {
+    print "Roots found: (", join(", ", @x), ")\n";
+  }
+  else
+  {
+    print "Problem root-finding for [", join(", ", @cubic), "]\n";
+  }
+
 =head3 laguerre()
 
-A numerical method for finding a root of an equation, especially made for polynomials.
+A numerical method for finding a root of an equation, especially made
+for polynomials.
 
-    @roots = laguerre(\@coefficients, \@xvalues);
-    push @roots, laguerre(\@coefficients, $another_xvalue);
+  @roots = laguerre(\@coefficients, \@xvalues);
+  push @roots, laguerre(\@coefficients, $another_xvalue);
 
 For each x value the function will attempt to find a root closest to it.
 
+=head3 newtonraphson()
+
+Like laguerre, a numerical method for finding a root of an equation.
+
+  @roots = laguerre(\@coefficients, \@xvalues);
+  push @roots, laguerre(\@coefficients, $another_xvalue);
+
+For each x value the function will attempt to find a root closest to it.
+
+This function is provided for comparisons purposes for the user; internally
+laguerre() is used.
+
 =head3 poly_iteration()
 
-Sets the limit to the number of iterations that may go by before giving up on
-finding a root. Each method of root-finding used by L</poly_roots()>, L</sturm_bisection_roots()>,
-and L</laguerre()> has its own iteration limit, which may be found, like L</poly_option()>, by
-simply looking at the return value of the function.
+Sets the limit to the number of iterations that a solving method may go
+through before giving up trying to find a root. Each method of root-finding
+used by L</poly_roots()>, L</sturm_bisection_roots()>, and L</laguerre()>
+has its own iteration limit, which may be found, like L</poly_option()>,
+simply by looking at the return value of poly_iteration().
 
-    #
-    # Get all of the current iteration limits.
-    #
-    my %its_limits = poly_iteration();
+  #
+  # Get all of the current iteration limits.
+  #
+  my %its_limits = poly_iteration();
 
-    #
-    # Double the limit for the hessenberg method, but set the limit
-    # for Laguerre's method to 20.
-    #
-    poly_iteration(hessenberg => $its_limits{hessenberg} * 2, laguerre => 12);
+  #
+  # Double the limit for the hessenberg method, but set the limit
+  # for Laguerre's method to 20.
+  #
+  poly_iteration(hessenberg => $its_limits{hessenberg} * 2, laguerre => 12);
 
-    #
-    # Reset the limits with the former values, but save the values we had
-    # for later.
-    #
-    my %hl_limits = poly_iteration(%its_limits);
+  #
+  # Reset the limits with the former values, but save the values we had
+  # for later.
+  #
+  my %hl_limits = poly_iteration(%its_limits);
 
-There are limit values for
+There are iteration limit values for:
 
 =over 4
 
 =item hessenberg
 
 The numeric method used by poly_roots(), if the hessenberg option is set.
+Its default value is 60.
+
+=item newtonraphson
+
+The numeric method used by newtonraphson(). The Newton-Raphson method is offered
+as an alternative to Laguerre's method.  Its default value is 60.
 
 =item laguerre
 
 The numeric method used by laguerre(). Laguerre's method is used within
-sturm_bisection_roots() once an individual root has been found within a range.
+sturm_bisection_roots() once it has narrowed its search in on an individual root,
+and of course laguerre() may be called independently. Its default value is
+60.
 
 =item sturm_bisection
 
-The bisection method used to find roots within a range.
+The bisection method used to find roots within a range. Its default value
+is 100.
 
 =back
 
-All of these items start with a default value of 60.
+=head3 poly_tolerance()
+
+Set the degree of accuracy needed for comparisons to be equal or roots to
+be found.  Amongst the root finding functions this currently only
+affects laguerre() and newtonraphson(), as the Hessenberg matrix method determines
+how close it needs to get using a complicated formula based on L</epsilon()>.
+
+  #
+  # Print the tolerances.
+  #
+  my %tolerances = poly_tolerance();
+  print "Default tolerances:\n";
+  foreach my $k (keys %tolerances)
+  {
+    print "$k => ", $tolerances{$k}, "\n";
+  }
+
+  #
+  # Quadruple the tolerance for Laguerre's method.
+  #
+  poly_tolerance(laguerre => 4 * $tolerances{laguerre});
+
+Tolerances may be set for:
+
+=over 4
+
+=item laguerre
+
+The numeric method used by laguerre(). Laguerre's method is used within
+sturm_bisection_roots() once an individual root has been found within a range,
+and of course it may be called independently.
+
+=item fltcmp
+
+A comparison function that determines if one argument is less than, equal to,
+or greater than, the other. Comparisons are made within a range determined by
+the tolerance.
+
+=back
 
 =head3 poly_derivative()
 
-    @derivative = poly_derivative(@coefficients);
+  @derivative = poly_derivative(@coefficients);
 
 Returns the coefficients of the first derivative of the polynomial.
 Leading zeros are removed before returning the derivative, so the length
@@ -2041,8 +2350,8 @@ polynomial. Returns an empty list if the polynomial is a simple constant.
 Returns the coefficients of the antiderivative of the polynomial. The
 constant term is set to zero; to override this use
 
-    @integral = poly_antiderivative(@coefficients);
-    $integral[$#integral] = $const_term;
+  @integral = poly_antiderivative(@coefficients);
+  $integral[$#integral] = $const_term;
 
 =head3 simplified_form()
 
@@ -2059,22 +2368,37 @@ as a reference.
 
 The function may return a list...
 
-    my @polynomial = (1, -12, 0, 8, 13);
-    my @xvals = (0, 1, 2, 3, 5, 7);
-    my @yvals = poly_evaluate(\@polynomial, \@xvals);
+  my @coefficients = (1, -12, 0, 8, 13);
+  my @xvals = (0, 1, 2, 3, 5, 7);
+  my @yvals = poly_evaluate(\@coefficients, \@xvals);
 
-    print "Polynomial: [", join(", ", @polynomial), "]\n";
+  print "Polynomial: [", join(", ", @coefficients), "]\n";
 
-    for my $j (0..$#yvals) {
-        print "Evaluates at ", $xvals[$j], " to ", $yvals[$j], "\n";
-    }
+  for my $j (0..$#yvals)
+  {
+    print "Evaluates at ", $xvals[$j], " to ", $yvals[$j], "\n";
+  }
 
 or return a scalar.
  
-    my $x_mean = ($xvals[0] + $xvals[$#xvals])/2.0;
-    my $y_mean = poly_evaluate(\@polynomial, $x_mean);
+  my $x_median = ($xvals[0] + $xvals[$#xvals])/2.0;
+  my $y_median = poly_evaluate(\@coefficients, $x_median);
 
 
+=head3 poly_derivaluate();
+
+Given an X value, returns the y-values of the polynomial, its first derivative,
+and its second derivative.
+
+  my($y, $dy, $ddy) = poly_derivaluate(\@coefficients, $x);
+
+Note that unlike L</poly_evaluate()>, this takes a single
+x-value.
+
+If the polynomial is a linear equation, the second derivative value will be
+zero.  Similarly, if the "equation" is a constant, the first derivative value
+will be zero.
+
 =head3 poly_nonzero_term_count()
 
 Returns a simple count of the number of coefficients that aren't zero.
@@ -2084,8 +2408,8 @@ Returns a simple count of the number of coefficients that aren't zero.
 Simple function to multiply all of the coefficients by a constant. Like
 C<poly_evaluate()>, uses the reference of the coefficient list.
 
-    my @polynomial = (1, 7, 0, 12, 19);
-    my @polynomial3 = poly_constmult(\@polynomial, 3);
+  my @coefficients = (1, 7, 0, 12, 19);
+  my @coef3 = poly_constmult(\@coefficients, 3);
 
 =head3 poly_divide()
 
@@ -2093,20 +2417,20 @@ Divide one polynomial by another. Like C<poly_evaluate()>, the function takes
 a reference to the coefficient list. It returns a reference to both a quotient
 and a remainder.
 
-    my @polynomial = (1, -13, 59, -87);
-    my @polydiv = (3, -26, 59);
-    my($q, $r) = poly_divide(\@polynomial, \@polydiv);
-    my @quotient = @$q;
-    my @remainder = @$r;
+  my @coefficients = (1, -13, 59, -87);
+  my @polydiv = (3, -26, 59);
+  my($q, $r) = poly_divide(\@coefficients, \@polydiv);
+  my @quotient = @$q;
+  my @remainder = @$r;
 
 =head1 EXPORT
 
 There are no default exports. The functions may be individually named in an
 export list, but there are also four export tags:
-L<classical|"Classical Functions">,
-L<numeric|"Numeric Functions">,
-L<sturm|"Sturm Functions">, and
-L<utility|"Utility Functions">.
+L<classical|Classical Functions>,
+L<numeric|Numeric Functions>,
+L<sturm|Sturm Functions>, and
+L<utility|Utility Functions>.
 
 =head1 ACKNOWLEDGMENTS
 
@@ -2206,7 +2530,7 @@ Acton, Forman S. I<Numerical Methods That Work>. New York: Harper & Row, Publish
 =over 5
 
 Lively, opinionated book on numerical equation solving. I looked it up when it
-became obvious that everone was quoting Acton when discussing Laguerre's method.
+became obvious that everyone was quoting Acton when discussing Laguerre's method.
 
 =back
 
